<?php

/*
 * Copyright 2013 Shaun Simmons
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Recurr\Exception;

use Recurr\Exception;

/**
 * @package Recurr\Exception
 * @author  Shaun Simmons <shaun@envysphere.com>
 */
class InvalidRRule extends Exception
{
}
